"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Object Path Parser
 * Keiya Chinen @ TwoGate inc.
 */
const typedgate_error_1 = require("./typedgate-error");
class ObjectPath {
    constructor(pathArg) {
        this.pathArg = pathArg;
        if (Array.isArray(pathArg)) {
            this.path = pathArg;
        }
        else if (typeof pathArg === 'string') {
            if (pathArg === '.') {
                this.path = [];
            }
            else {
                if (pathArg[0] === '.') {
                    this.path = ObjectPath.toArray(pathArg.substr(1));
                }
                else {
                    throw new typedgate_error_1.TypedgateError('Path should be start with `.`');
                }
            }
        }
        else if (isNaN(pathArg)) {
            this.path = [pathArg];
        }
        else {
            throw new typedgate_error_1.TypedgateError('Wrong Argument');
        }
    }
    static toArray(pathStr) {
        const arrayRex = new RegExp(/(.+)\[(\d+)\]$/);
        const emptyArrayRex = new RegExp(/(.*)\[\]$/);
        const splited = pathStr.split('.');
        const res = splited.reduce((acc, r) => {
            const arrRex = r.match(arrayRex);
            const emptyArrRex = r.match(emptyArrayRex);
            if (arrRex) {
                acc = acc.concat([arrRex[1], parseInt(arrRex[2], 10)]);
            }
            else if (emptyArrRex) {
                if (emptyArrRex[1]) {
                    acc = acc.concat([emptyArrRex[1], []]);
                }
                else {
                    acc.push([]);
                }
            }
            else {
                acc.push(r);
            }
            return acc;
        }, []);
        return res;
    }
    traverse(data) {
        let lastElmIsEmptyArray = false;
        return this.path.reduce((previous, current) => {
            if (typeof current === 'number' || typeof current === 'string') {
                if (lastElmIsEmptyArray) {
                    lastElmIsEmptyArray = false;
                    return previous.map((p) => p[current]);
                }
                return previous[current];
            }
            else if (current instanceof Array && current.length === 0) {
                lastElmIsEmptyArray = true;
                return previous;
            }
        }, data);
    }
}
exports.ObjectPath = ObjectPath;
//# sourceMappingURL=object-path.js.map